/*
 * Copyright 2008 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */
#include "Object.h"
#include <map>
#include <iostream>

namespace FD {

template <class _Key, class _Tp>
class Map : public Object , public std::map<_Key, _Tp> {
public:
	Map() {}
	Map(const Map &m) : std::map<_Key, _Tp> (m) {}

   /**
      Formatted Map output in the FlowDesigner Format.<br>
      <b>Format : </b> \<Map\<T\> <i> channel0 channel1 ... channel(size - 1)</i> \>
      \param out the output stream
   */
	/*virtual*/ void printOn(std::ostream& out = std::cout) const {}

	/**
	   Clone the vector and return an identical copy (deep copy)
	*/
	/*virtual*/ ObjectRef clone() {};
};

///printOn
template <>
inline void Map<int, ObjectRef>::printOn(std::ostream& out) const {
	if (this->empty() == true) {
		out << "Map empty" << std::endl;
	}
	else {
		Map<int, ObjectRef>::const_iterator it = this->begin();
		out << "Map begins from " << (it->first) << std::endl;
		out << "Number of channels is " << this->size() << std::endl;
		for (; it != this->end(); ++it) {
			out << "Map's ObjectRef's channel " << (it->first) << " is:\n" << (it->second) << std::endl;
		}
	}
}

///clone
template <>
inline ObjectRef Map<int, ObjectRef>::clone() {
	ObjectRef cpy = ObjectRef(new Map<int, ObjectRef>);
	Map<int, ObjectRef> &mpOr = object_cast<Map<int, ObjectRef> > (cpy);
	for (Map<int, ObjectRef>::const_iterator it = this->begin(); it != this->end(); ++it) {
		mpOr.insert(std::make_pair(it->first, it->second));
	}
	return cpy;

    /*ObjectRef cpy(this);
	Map<int, ObjectRef> &mpOr = object_cast<Map<int, ObjectRef> > (cpy);
	for (Map<int, ObjectRef>::const_iterator it = mpOr.begin(); it != mpOr.end(); ++it) {
		//it->second = it->second->clone();
		mpOr.insert(std::make_pair(it->first, it->second));
	}
    return cpy;*/
}

}//end namespace FD
